@@ -1,15 +1,16 @@
 #! /bin/sh
 
-# Copyright (C) 2001-2007 Peter Selinger.
+# Copyright (C) 2001-2010 Peter Selinger.
 # This file is part of Potrace. It is free software and it is covered
 # by the GNU General Public License. See the file COPYING for details.
 
+echo "Checking backends..." >& 2
+
 if test "$#" -gt 0 -a "$1" = "-t"; then
     testflag=yes
+    shift
 fi
 
-echo "Checking backends..." >& 2
-
 # we cannot check the output exactly because of floating point
 # differences on different architectures. So we content ourselves with
 # checking the file type and the approximate file size. Note that the
@@ -25,7 +26,7 @@ fi
 
 NAME=`basename "$0"`
 
-POTRACE="../src/potrace --progress"
+POTRACE="${CHECK_POTRACE:-../src/potrace$EXEEXT --progress}"
 TMPDIR="${TEMPDIR:-/tmp}"
 TMP1=`mktemp "$TMPDIR/$NAME-1.XXXXXX"`
 DATA="$srcdir/data1.pbm"
@@ -96,21 +97,15 @@ action filetest "$TMP1" '^%!PS-Adobe-3.0 EPSF-3.0$' > /dev/null
 action sizetest "$TMP1" 9000 9800
 
 # test eps with level 2 compression
-R=`action $POTRACE -2 -o "$TMP1" "$DATA" 2>&1`
-if test "$?" -ne 0; then
-    exit
-fi
-if echo "$R" | grep 'option -2 not supported' > /dev/null ; then
-  true # do nothing
-else
-  action filetest "$TMP1" '^%!PS-Adobe-3.0 EPSF-3.0$' > /dev/null
-  action sizetest "$TMP1" 5500 5900
-fi
+action $POTRACE -2 -o "$TMP1" "$DATA"
+action filetest "$TMP1" '^%!PS-Adobe-3.0 EPSF-3.0$' > /dev/null
+action sizetest "$TMP1" 5500 5900
 
 # test eps with level 3 compression
 R=`action $POTRACE -3 -o "$TMP1" "$DATA" 2>&1`
 if test "$?" -ne 0; then
-    exit
+    echo "$R" >& 2
+    exit 1
 fi
 if echo "$R" | grep 'option -3 not supported' > /dev/null ; then
   true # do nothing
@@ -144,7 +139,7 @@ action sizetest "$TMP1" 9000 9850
 # test pdf
 action $POTRACE -b pdf -c -o "$TMP1" "$DATA"
 action filetest "$TMP1" '^%PDF-1.3$' > /dev/null
-action sizetest "$TMP1" 17000 18000
+action sizetest "$TMP1" 16800 18000
 
 # test svg
 action $POTRACE -b svg -o "$TMP1" "$DATA"
@@ -154,7 +149,7 @@ action sizetest "$TMP1" 12000 13850
 # test svg debugging backend
 action $POTRACE -b svg -d1 -o "$TMP1" "$DATA"
 action filetest "$TMP1" '^<\?xml version="1.0" standalone="no"\?>$' > /dev/null
-action sizetest "$TMP1" 21100 21500
+action sizetest "$TMP1" 21050 21500
 
 # test pgm
 action $POTRACE -b pgm -o "$TMP1" "$DATA"
