@@ -1,6 +1,6 @@
 #! /bin/sh
 
-# Copyright (C) 2001-2007 Peter Selinger.
+# Copyright (C) 2001-2010 Peter Selinger.
 # This file is part of Potrace. It is free software and it is covered
 # by the GNU General Public License. See the file COPYING for details.
 
@@ -17,7 +17,7 @@ fi
 
 NAME=`basename "$0"`
 
-POTRACE="../src/potrace --progress"
+POTRACE="${CHECK_POTRACE:-../src/potrace$EXEEXT --progress}"
 TMPDIR="${TEMPDIR:-/tmp}"
 TMP1=`mktemp "$TMPDIR/$NAME-1.XXXXXX"`
 TMP2=`mktemp "$TMPDIR/$NAME-2.XXXXXX"`
@@ -35,7 +35,7 @@ action () {
 alias action="LINE=\$LINENO; action"
 
 # available input files
-INFILES="data1.pbm data1.pgm data1.ppm data1.bmp1 data1.bmp4 data1.bmp8 data1.bmp24 data1.bmp4r data1.bmp8r"
+INFILES="data1.pbm data1.pgm data1.ppm data1.bmp1 data1.bmp4 data1.bmp8 data1.bmp24 data1.bmp24td data1.bmp24v5 data1.bmp4r data1.bmp8r"
 
 # extract first file
 set dummy $INFILES; F="$2"
