@@ -1,6 +1,6 @@
 #! /bin/sh
 
-# Copyright (C) 2001-2005 Peter Selinger.
+# Copyright (C) 2001-2010 Peter Selinger.
 # This file is free software and it is covered by the GNU general
 # public license. See the file COPYING for details.
 
@@ -10,7 +10,7 @@
 # we need to implement our own. This code is adapted from autoconf.
 
 my_which () {
-    if [ "$#" -ne 1 ]; then
+    if test "$#" -ne 1; then
 	echo "my_which: wrong number of arguments" >&2
 	return 255
     fi
@@ -33,11 +33,11 @@ my_which () {
 # Thus, when creating more than one tempfile, must give different templates.
 
 MKTEMP=`my_which mktemp`
-if [ -z "$MKTEMP" ]; then
+if test -z "$MKTEMP"; then
     echo "Warning: the mktemp program is missing. Using a dummy replacement." >&2
     mktemp () {
 	echo "$1" | sed -e "s/XXXXXX/$$/"
     }
 fi
 
-	
\ No newline at end of file
+	
