@@ -1,6 +1,6 @@
 #! /bin/sh
 
-# Copyright (C) 2001-2007 Peter Selinger.
+# Copyright (C) 2001-2010 Peter Selinger.
 # This file is part of Potrace. It is free software and it is covered
 # by the GNU General Public License. See the file COPYING for details.
 
@@ -18,8 +18,8 @@ fi
 
 NAME=`basename "$0"`
 
-POTRACE="../src/potrace --progress"
-PGMDIFF=./pgmdiff
+POTRACE="${CHECK_POTRACE:-../src/potrace$EXEEXT --progress}"
+PGMDIFF="./pgmdiff$EXEEXT"
 TMPDIR="${TEMPDIR:-/tmp}"
 TMP1=`mktemp "$TMPDIR/$NAME-1.XXXXXX"`
 DATA="$srcdir/data1.pbm"
