@@ -1,10 +1,10 @@
-/* Copyright (C) 2001-2005 Peter Selinger.
-   This file is part of potrace. It is free software and it is covered
+/* Copyright (C) 2001-2010 Peter Selinger.
+   This file is part of Potrace. It is free software and it is covered
    by the GNU General Public License. See the file COPYING for details. */
 
-/* $Id: backend_eps.c,v 1.23 2005/02/22 21:31:23 selinger Exp $ */
+/* $Id: backend_eps.c 227 2010-12-16 05:47:19Z selinger $ */
 
-/* The Postscript backend of potrace. This can produce "ps" or "eps"
+/* The Postscript backend of Potrace. This can produce "ps" or "eps"
    output, and different kinds of graphical debugging
    output. Postscript compression is optionally supplied via the
    functions in flate.c. */
@@ -305,10 +305,10 @@ static int eps_path_short(privcurve_t *curve) {
 }
 
 static int eps_path(privcurve_t *curve) {
-  if (info.longcoding) {
-    return eps_path_long(curve);
-  } else {
+  if (info.longcoding==0 && curve->alphacurve) {
     return eps_path_short(curve);
+  } else {
+    return eps_path_long(curve);
   }
 }
 
@@ -685,7 +685,7 @@ static int eps_init(imginfo_t *imginfo) {
   char *c0, *c1;
 
   shipcom("%%!PS-Adobe-3.0 EPSF-3.0\n");
-  shipcom("%%%%Creator: "POTRACE" "VERSION", written by Peter Selinger 2001-2005\n");
+  shipcom("%%%%Creator: "POTRACE" "VERSION", written by Peter Selinger 2001-2010\n");
   shipcom("%%%%LanguageLevel: %d\n", info.pslevel);
   shipcom("%%%%BoundingBox: 0 0 %d %d\n", 
 	  (int)ceil(imginfo->trans.bb[0]+imginfo->lmar+imginfo->rmar),
@@ -694,9 +694,15 @@ static int eps_init(imginfo_t *imginfo) {
   shipcom("%%%%EndComments\n");
   
   shipcom("%%%%Page: 1 1\n");
+  ship("save\n");
   if (!info.longcoding) {
     c0 = strdup(eps_colorstring(info.color));
     c1 = strdup(eps_colorstring(info.fillcolor));
+    if (!c0 || !c1) {
+      free(c0);
+      free(c1);
+      return 1;
+    }
     ship(optimacros, c0, c1);
     free(c0);
     free(c1);
@@ -704,7 +710,6 @@ static int eps_init(imginfo_t *imginfo) {
   if (info.debug) {
     ship(debugmacros, info.unit);
   }
-  ship("gsave\n");
   if (origx != 0 || origy != 0) {
     ship("%.0f %.0f translate\n", origx, origy);
   }
@@ -716,8 +721,8 @@ static int eps_init(imginfo_t *imginfo) {
   return 0;
 }
 
-static int eps_term() {
-  ship("grestore\n");
+static int eps_term(void) {
+  ship("restore\n");
   shipcom("%%%%EOF\n");
   return 0;
 }
@@ -752,7 +757,7 @@ int init_ps(FILE *fout) {
   eps_callbacks(fout);
 
   shipcom("%%!PS-Adobe-3.0\n");
-  shipcom("%%%%Creator: "POTRACE" "VERSION", written by Peter Selinger 2001-2005\n");
+  shipcom("%%%%Creator: "POTRACE" "VERSION", written by Peter Selinger 2001-2010\n");
   shipcom("%%%%LanguageLevel: %d\n", info.pslevel);
   shipcom("%%%%BoundingBox: 0 0 %d %d\n", info.paperwidth, info.paperheight);
   shipcom("%%%%Pages: (atend)\n");
@@ -762,6 +767,11 @@ int init_ps(FILE *fout) {
     if (!info.longcoding) {
       c0 = strdup(eps_colorstring(info.color));
       c1 = strdup(eps_colorstring(info.fillcolor));
+      if (!c0 || !c1) {
+	free(c0);
+	free(c1);
+	return 1;
+      }
       ship(optimacros, c0, c1);
       free(c0);
       free(c1);
@@ -798,7 +808,7 @@ static void eps_pageinit_ps(imginfo_t *imginfo) {
   eps_width = -1;
 
   shipcom("%%%%Page: %d %d\n", eps_pagenumber, eps_pagenumber);
-  ship("gsave\n");
+  ship("save\n");
   if (origx != 0 || origy != 0) {
     ship("%.0f %.0f translate\n", origx, origy);
   }
@@ -808,8 +818,8 @@ static void eps_pageinit_ps(imginfo_t *imginfo) {
   ship("%f %f scale\n", scalex, scaley);
 }
 
-static void eps_pageterm_ps() {
-  ship("grestore\n");
+static void eps_pageterm_ps(void) {
+  ship("restore\n");
   ship("showpage\n");
 }
 
