@@ -1,8 +1,8 @@
-/* Copyright (C) 2001-2007 Peter Selinger.
+/* Copyright (C) 2001-2010 Peter Selinger.
    This file is part of Potrace. It is free software and it is covered
    by the GNU General Public License. See the file COPYING for details. */
 
-/* $Id: bitmap_io.c 147 2007-04-09 00:44:09Z selinger $ */
+/* $Id: bitmap_io.c 227 2010-12-16 05:47:19Z selinger $ */
 
 /* Routines for manipulating bitmaps, including reading pbm files. */
 
@@ -348,6 +348,7 @@ struct bmp_info_s {
   unsigned int ncolors;        /* number of colors in palette */
   unsigned int ColorsImportant;
   unsigned int ctbits;         /* sample size for color table */
+  int topdown;                 /* top-down mode? */
 };
 typedef struct bmp_info_s bmp_info_t;
 
@@ -416,6 +417,9 @@ static int bmp_forward(FILE *f, int pos) {
 #define TRY(x) if (x) goto try_error
 #define TRY_EOF(x) if (x) goto eof
 
+/* correct y-coordinate for top-down format */
+#define ycorr(y) (bmpinfo.topdown ? bmpinfo.h-1-y : y)
+
 /* read BMP stream after magic number. Return values as for bm_read.
    We choose to be as permissive as possible, since there are many
    programs out there which produce BMP. For instance, ppmtobmp can
@@ -449,7 +453,8 @@ static int bm_readbody_bmp(FILE *f, double threshold, potrace_bitmap_t **bmp) {
 
   /* info header */
   TRY(bmp_readint(f, 4, &bmpinfo.InfoSize));
-  if (bmpinfo.InfoSize == 40 || bmpinfo.InfoSize == 64) {
+  if (bmpinfo.InfoSize == 40 || bmpinfo.InfoSize == 64
+      || bmpinfo.InfoSize == 108 || bmpinfo.InfoSize == 124) {
     /* Windows or new OS/2 format */
     bmpinfo.ctbits = 32; /* sample size in color table */
     TRY(bmp_readint(f, 4, &bmpinfo.w));
@@ -462,6 +467,12 @@ static int bm_readbody_bmp(FILE *f, double threshold, potrace_bitmap_t **bmp) {
     TRY(bmp_readint(f, 4, &bmpinfo.YpixelsPerM));
     TRY(bmp_readint(f, 4, &bmpinfo.ncolors));
     TRY(bmp_readint(f, 4, &bmpinfo.ColorsImportant));
+    if ((signed int)bmpinfo.h < 0) {
+      bmpinfo.h = -bmpinfo.h;
+      bmpinfo.topdown = 1;
+    } else {
+      bmpinfo.topdown = 0;
+    }
   } else if (bmpinfo.InfoSize == 12) {
     /* old OS/2 format */
     bmpinfo.ctbits = 24; /* sample size in color table */
@@ -471,11 +482,12 @@ static int bm_readbody_bmp(FILE *f, double threshold, potrace_bitmap_t **bmp) {
     TRY(bmp_readint(f, 2, &bmpinfo.bits));
     bmpinfo.comp = 0;
     bmpinfo.ncolors = 0;
+    bmpinfo.topdown = 0;
   } else {
     goto format_error;
   }
 
-  /* forward to color table (i.e., if bmpinfo.InfoSize == 64) */
+  /* forward to color table (e.g., if bmpinfo.InfoSize == 64) */
   TRY(bmp_forward(f, 14+bmpinfo.InfoSize));
 
   if (bmpinfo.Planes != 1) {
@@ -538,7 +550,7 @@ static int bm_readbody_bmp(FILE *f, double threshold, potrace_bitmap_t **bmp) {
       for (i=0; 8*i<bmpinfo.w; i++) {
 	TRY_EOF(bmp_readint(f, 1, &b));
 	b ^= mask;
-	*bm_index(bm, i*8, y) |= ((potrace_word)b) << (8*(BM_WORDSIZE-1-(i % BM_WORDSIZE)));
+	*bm_index(bm, i*8, ycorr(y)) |= ((potrace_word)b) << (8*(BM_WORDSIZE-1-(i % BM_WORDSIZE)));
       }
       TRY(bmp_pad(f));
     }
@@ -564,7 +576,7 @@ static int bm_readbody_bmp(FILE *f, double threshold, potrace_bitmap_t **bmp) {
 	b = bitbuf >> (INTBITS - bmpinfo.bits);
 	bitbuf <<= bmpinfo.bits;
 	n -= bmpinfo.bits;
-	BM_UPUT(bm, x, y, coltable[b]);
+	BM_UPUT(bm, x, ycorr(y), coltable[b]);
       }
       TRY(bmp_pad(f));
     }
@@ -584,7 +596,7 @@ static int bm_readbody_bmp(FILE *f, double threshold, potrace_bitmap_t **bmp) {
       for (x=0; x<bmpinfo.w; x++) {
         TRY_EOF(bmp_readint(f, bmpinfo.bits/8, &c));
 	c = ((c>>16) & 0xff) + ((c>>8) & 0xff) + (c & 0xff);
-        BM_UPUT(bm, x, y, c > 3 * threshold * 255 ? 0 : 1);
+        BM_UPUT(bm, x, ycorr(y), c > 3 * threshold * 255 ? 0 : 1);
       }
       TRY(bmp_pad(f));
     }
@@ -608,7 +620,7 @@ static int bm_readbody_bmp(FILE *f, double threshold, potrace_bitmap_t **bmp) {
 	  if (y>=bmpinfo.h) {
 	    break;
 	  }
-	  BM_UPUT(bm, x, y, col[i&1]);
+	  BM_UPUT(bm, x, ycorr(y), col[i&1]);
 	  x++;
 	}
       } else if (c == 0) {
@@ -637,7 +649,7 @@ static int bm_readbody_bmp(FILE *f, double threshold, potrace_bitmap_t **bmp) {
 	  if (y>=bmpinfo.h) {
 	    break;
 	  }
-	  BM_PUT(bm, x, y, coltable[(b>>(4-4*(i&1))) & 0xf]);
+	  BM_PUT(bm, x, ycorr(y), coltable[(b>>(4-4*(i&1))) & 0xf]);
 	  x++;
 	}
 	if ((c+1) & 2) {
@@ -664,7 +676,7 @@ static int bm_readbody_bmp(FILE *f, double threshold, potrace_bitmap_t **bmp) {
 	  if (y>=bmpinfo.h) {
 	    break;
 	  }
-	  BM_UPUT(bm, x, y, coltable[c]);
+	  BM_UPUT(bm, x, ycorr(y), coltable[c]);
 	  x++;
 	}
       } else if (c == 0) {
@@ -691,7 +703,7 @@ static int bm_readbody_bmp(FILE *f, double threshold, potrace_bitmap_t **bmp) {
           if (y>=bmpinfo.h) {
             break;
           }
-	  BM_PUT(bm, x, y, coltable[b]);
+	  BM_PUT(bm, x, ycorr(y), coltable[b]);
 	  x++;
 	}
 	if (c & 1) {
