@@ -1,11 +1,11 @@
-/* Copyright (C) 2001-2007 Peter Selinger.
+/* Copyright (C) 2001-2010 Peter Selinger.
    This file is part of Potrace. It is free software and it is covered
    by the GNU General Public License. See the file COPYING for details. */
 
-/* $Id: greymap.h 147 2007-04-09 00:44:09Z selinger $ */
+/* $Id: greymap.h 227 2010-12-16 05:47:19Z selinger $ */
 
-#ifndef PGM_H
-#define PGM_H
+#ifndef GREYMAP_H
+#define GREYMAP_H
 
 #include <stdio.h>
 
@@ -55,4 +55,4 @@ int gm_read(FILE *f, greymap_t **gmp);
 int gm_writepgm(FILE *f, greymap_t *gm, char *comment, int raw, int mode, double gamma);
 int gm_print(FILE *f, greymap_t *gm);
 
-#endif /* PGM_H */
+#endif /* GREYMAP_H */
