@@ -1,8 +1,8 @@
-/* Copyright (C) 2001-2005 Peter Selinger.
-   This file is part of potrace. It is free software and it is covered
+/* Copyright (C) 2001-2010 Peter Selinger.
+   This file is part of Potrace. It is free software and it is covered
    by the GNU General Public License. See the file COPYING for details. */
 
-/* $Id: lzw.c,v 1.6 2005/02/22 21:31:23 selinger Exp $ */
+/* $Id: lzw.c 247 2010-12-21 14:54:10Z selinger $ */
 
 /* code for adaptive LZW compression, as used in PostScript. */
 
@@ -23,8 +23,8 @@
    is a string of input symbols, d is a dictionary, which is a
    function from strings to output symbols, and n is the dictionary
    size, or equivalently, the next unused output symbol. There are
-   also special states init and stop. (emit.c[b]ode) is a function
-   which emits the code code as a b-bit value into the output
+   also special states init and stop. emit[b, code] is a function
+   which emits the code 'code' as a b-bit value into the output
    bitstream. hibit(n) returns the least number of binary digits
    required to represent n.
 
@@ -35,29 +35,28 @@
      output symbols 256 and 257, so that the next available one is
      258). Note: hibit(258)=9.
 
-   {[], d, n} (input c) ---> (emit[hibit(n)] 256) {c, d, n}
+   {[], d, n} (input c) ---> (emit[hibit(n), 256]) {c, d, n}
 
    {s,d,n} (input c) ---> {s*c,d,n}
 
      if s!=[], s*c is in the domain of d. Here s*c is the strings s
      extended by the character c.
 
-   {s,d,n} (input c) ---> (emit.c[hibit(n)]ode) {c,d',n+1}
+   {s,d,n} (input c) ---> (emit[hibit(n), d(s)]) {c,d',n+1}
 
      if s!=[], s*c is not in the domain of d, and hibit(n+2) <= 12.
-     Here, d(s)=code, and d'=d+{s*c->n}.
+     Here d'=d+{s*c->n}.
 
    {s,d,n} (input c) ---> 
-           (emit.c[hibit(n)]ode) (emit[hibit(n+1)] 256) {c, newdict, 258}
+           (emit[hibit(n), d(s)]) (emit[hibit(n+1), 256]) {c, newdict, 258}
 
      if s!=[], s*c is not in the domain of d, and hibit(n+2) > 12.
-     Here, d(s)=code, and d'=d+{s*c->n}.
 
-   {s,d,n} (input EOD) ---> (emit.c[hibit(n)]ode) (emit[hibit(n+1)] 257) stop
+   {s,d,n} (input EOD) ---> (emit[hibit(n), d(s)]) (emit[hibit(n+1), 257]) stop
 
-     where s != [], code = d(s). Here, EOD stands for end-of-data.
+     where s != []. Here, EOD stands for end-of-data.
 
-   {[],d,n} (input EOD) ---> (emit[hibit(n)] 256) (emit[hibit(258)] 257) stop
+   {[],d,n} (input EOD) ---> (emit[hibit(n), 256]) (emit[hibit(n), 257]) stop
 
    Notes: 
 
@@ -77,47 +76,47 @@
 
 /* the dictionary is implemented as a tree of strings under the prefix
    order. The tree is in turns represented as a linked list of
-   lzw_dict structures, with "children" pointing to a node's first
+   lzw_dict_t structures, with "children" pointing to a node's first
    child, and "next" pointing to a node's next sibling. As an
    optimization, the top-level nodes (singleton strings) are
    implemented lazily, i.e., the corresponding entry is not actually
    created until it is accessed. */
 
-struct lzw_dict {
+struct lzw_dict_s {
   char c;      /* last character of string represented by this entry */
   int code;    /* code for the string represented by this entry */
   int freq;    /* how often searched? For optimization only */
-  struct lzw_dict *children;  /* list of sub-entries */
-  struct lzw_dict *next;      /* for making a linked list */
+  struct lzw_dict_s *children;  /* list of sub-entries */
+  struct lzw_dict_s *next;      /* for making a linked list */
 };
-typedef struct lzw_dict lzw_dict;
+typedef struct lzw_dict_s lzw_dict_t;
 
 /* A state {s,d,n} is represented by the "dictionary state" part of
-   the lzw_state structure. Here, s is a pointer directly to the node
+   the lzw_state_t structure. Here, s is a pointer directly to the node
    of the dictionary structure corresponding to the string s, or NULL
-   if s=[]. Further, the lzw_state structure contains a buffer of
+   if s=[]. Further, the lzw_state_t structure contains a buffer of
    pending output bits, and a flag indicating whether the EOD (end of
    data) has been reached in the input. */
 
-struct lzw_state {
+struct lzw_state_s {
   /* dictionary state */
   int n;           /* current size of the dictionary */
-  lzw_dict *d;     /* pointer to dictionary */
-  lzw_dict *s;     /* pointer to current string, or NULL at beginning */
+  lzw_dict_t *d;     /* pointer to dictionary */
+  lzw_dict_t *s;     /* pointer to current string, or NULL at beginning */
 
   /* buffers for pending output */
   BITBUF_TYPE buf; /* bits scheduled for output - left aligned, 0 padded */
   int bufsize;     /* number of bits scheduled for output. */
   int eod;         /* flush buffer? */
 };
-typedef struct lzw_state lzw_state;
+typedef struct lzw_state_s lzw_state_t;
 
 /* ---------------------------------------------------------------------- */
 /* auxiliary functions which operate on dictionary states */
 
-/* recursively free an lzw_dict object */
-static void lzw_free_dict(lzw_dict *s) {
-  lzw_dict *e;
+/* recursively free an lzw_dict_t object */
+static void lzw_free_dict(lzw_dict_t *s) {
+  lzw_dict_t *e;
 
   list_forall_unlink(e, s) {
     lzw_free_dict(e->children);
@@ -125,9 +124,9 @@ static void lzw_free_dict(lzw_dict *s) {
   }
 }
 
-/* re-initialize the lzw_state's dictionary state to "newdict",
+/* re-initialize the lzw state's dictionary state to "newdict",
    freeing any old dictionary. */
-static void lzw_clear_table(lzw_state *st) {
+static void lzw_clear_table(lzw_state_t *st) {
 
   lzw_free_dict(st->d);
   st->d = NULL;
@@ -141,7 +140,7 @@ static void lzw_clear_table(lzw_state *st) {
 /* write the code to the bit buffer. Precondition st->bufsize <= 7.
    Note: this does not change the dictionary state; in particular,
    n must be updated between consecutive calls. */
-static inline void lzw_emit(int code, lzw_state *st) {
+static inline void lzw_emit(int code, lzw_state_t *st) {
   BITBUF_TYPE mask;
   int bits = hibit(st->n);
 
@@ -157,7 +156,7 @@ static inline void lzw_emit(int code, lzw_state *st) {
    s->avail_out > 0. */
 static inline void lzw_read_bitbuf(lzw_stream_t *s) {
   int ch;
-  lzw_state *st = (lzw_state *)s->internal;
+  lzw_state_t *st = (lzw_state_t *)s->internal;
 
   ch = st->buf >> (8*sizeof(BITBUF_TYPE)-8);
   st->buf <<= 8;
@@ -175,12 +174,12 @@ static inline void lzw_read_bitbuf(lzw_stream_t *s) {
    ch. This updates the dictionary state and/or writes to the bit
    buffer. Precondition: st->bufsize <= 7. Return 0 on success, or 1
    on error with errno set. */
-static int lzw_encode_char(lzw_state *st, char c) {
-  lzw_dict *e;
+static int lzw_encode_char(lzw_state_t *st, char c) {
+  lzw_dict_t *e;
 
   /* st = {s,d,n}. hibit(n+1)<=12. */
 
-  /* {[], d, n} (input c) ---> (emit[hibit(n)] 256) {c, d, n} */
+  /* {[], d, n} (input c) ---> (emit[hibit(n), 256]) {c, d, n} */
   if (st->s == NULL) {
     lzw_emit(256, st);
     goto singleton;  /* enter singleton state c */
@@ -194,19 +193,19 @@ static int lzw_encode_char(lzw_state *st, char c) {
     return 0;
   }
 
-  /* {s,d,n} (input c) ---> (emit.c[hibit(n)]ode) {c,d',n+1} */
+  /* {s,d,n} (input c) ---> (emit[hibit(n), d(s)]) {c,d',n+1} */
   /* {s,d,n} (input c) --->
-	     (emit.c[hibit(n)]ode) (emit[hibit(n+1)] 256) {c, newdict, 258} */
+	    (emit[hibit(n), d(s)]) (emit[hibit(n+1), 256]) {c, newdict, 258} */
 
   lzw_emit(st->s->code, st); /* 9-12 bits */
-  if (st->n >= 4094) {   /* hibit(n+2) > 12 */  /* ### was: 4093 */
+  if (st->n >= 4094) {   /* hibit(n+2) > 12 */
     st->n++;
     lzw_emit(256, st);
     goto dictfull;    /* reset dictionary and enter singleton state c */
   }
 
   /* insert new code in dictionary, if possible */
-  e = (lzw_dict *)malloc(sizeof(lzw_dict));
+  e = (lzw_dict_t *)malloc(sizeof(lzw_dict_t));
   if (!e) {
     return 1;
   }
@@ -225,7 +224,7 @@ static int lzw_encode_char(lzw_state *st, char c) {
  singleton:   /* enter singleton state c */
   list_find(e, st->d, e->c == c);
   if (!e) {  /* not found: lazily add it */
-    e = (lzw_dict *)malloc(sizeof(lzw_dict));
+    e = (lzw_dict_t *)malloc(sizeof(lzw_dict_t));
     if (!e) {
       return 1;
     }
@@ -244,10 +243,10 @@ static int lzw_encode_char(lzw_state *st, char c) {
    the dictionary state undefined and writes to the bit buffer.
    Precondition: st->bufsize <= 7. This function must be called
    exactly once, at the end of the stream. */
-static void lzw_encode_eod(lzw_state *st) {
+static void lzw_encode_eod(lzw_state_t *st) {
 
   /* {[],d,n} (input EOD) ---> 
-              (emit[hibit(n)] 256) (emit[hibit(n+1)] 257) stop */
+              (emit[hibit(n), 256]) (emit[hibit(n), 257]) stop */
   if (st->s == NULL) {
     lzw_emit(256, st);  /* 9 bits */
     st->n=258;
@@ -256,7 +255,7 @@ static void lzw_encode_eod(lzw_state *st) {
   } 
 
   /* {s,d,n} (input EOD) ---> 
-             (emit.c[hibit(n)]ode) (emit[hibit(n+1)] 257) stop */
+             (emit[hibit(n), code]) (emit[hibit(n+1), 257]) stop */
 
   lzw_emit(st->s->code, st); /* 9-12 bits */
   st->n++;
@@ -270,13 +269,13 @@ static void lzw_encode_eod(lzw_state *st) {
 
 lzw_stream_t *lzw_init(void) {
   lzw_stream_t *s = NULL;
-  lzw_state *st = NULL;
+  lzw_state_t *st = NULL;
 
   s = (lzw_stream_t *)malloc(sizeof(lzw_stream_t));
   if (s==NULL) {
     goto fail;
   }
-  st = (lzw_state *)malloc(sizeof(lzw_state));
+  st = (lzw_state_t *)malloc(sizeof(lzw_state_t));
   if (st==NULL) {
     goto fail;
   }
@@ -296,7 +295,7 @@ lzw_stream_t *lzw_init(void) {
 
 int lzw_compress(lzw_stream_t *s, int mode) {
   int r;
-  lzw_state *st = (lzw_state *)s->internal;
+  lzw_state_t *st = (lzw_state_t *)s->internal;
 
   while (st->eod == 0) {
     /* empty bit buffer */
@@ -343,7 +342,7 @@ int lzw_compress(lzw_stream_t *s, int mode) {
 }
 
 void lzw_free(lzw_stream_t *s) {
-  lzw_state *st = (lzw_state *)s->internal;
+  lzw_state_t *st = (lzw_state_t *)s->internal;
 
   lzw_free_dict(st->d);
   free(st);
@@ -353,7 +352,7 @@ void lzw_free(lzw_stream_t *s) {
 /* ---------------------------------------------------------------------- */
 /* main function for testing and illustration purposes */
 
-#ifdef MAIN
+#ifdef LZW_MAIN
 
 int main() {
   lzw_stream_t *s;
