@@ -1,5 +1,5 @@
-/* Copyright (C) 2001-2005 Peter Selinger.
-   This file is part of potrace. It is free software and it is covered
+/* Copyright (C) 2001-2010 Peter Selinger.
+   This file is part of Potrace. It is free software and it is covered
    by the GNU General Public License. See the file COPYING for details. */
 
 /* A simple and self-contained demo of the potracelib API */
@@ -51,6 +51,14 @@ static potrace_bitmap_t *bm_new(int w, int h) {
   return bm;
 }
 
+/* free a bitmap */
+static void bm_free(potrace_bitmap_t *bm) {
+  if (bm != NULL) {
+    free(bm->map);
+  }
+  free(bm);
+}
+
 /* ---------------------------------------------------------------------- */
 /* demo */
 
@@ -91,6 +99,7 @@ int main() {
     fprintf(stderr, "Error tracing bitmap: %s\n", strerror(errno));
     return 1;
   }
+  bm_free(bm);
   
   /* output vector data, e.g. as a rudimentary EPS file */
   printf("%%!PS-Adobe-3.0 EPSF-3.0\n");
@@ -128,9 +137,8 @@ int main() {
   printf("grestore\n");
   printf("%%EOF\n");
   
+  potrace_state_free(st);
+  potrace_param_free(param);
+
   return 0;
 }
-
-  
-	
-
